@@ -25,8 +25,9 @@
 #include "config.h"
 
 #include "gdk/gdk.h"
-#include "gdk/gdk-private.h"
+#include "gdk/gdkdisplayprivate.h"
 #include "gdk/gdkprofilerprivate.h"
+#include "gdk/gdkdebugprivate.h"
 #include "gsk/gskprivate.h"
 #include "gsk/gskrendernodeprivate.h"
 #include "gtknative.h"
@@ -47,9 +48,10 @@
 #undef STRICT
 #endif
 
-#include "gtkintl.h"
+#include <hb-glib.h>
+
+#include <glib/gi18n-lib.h>
 
-#include "gtkbox.h"
 #include "gtkdebug.h"
 #include "gtkdropprivate.h"
 #include "gtkmain.h"
@@ -57,12 +59,11 @@
 #include "gtkmodulesprivate.h"
 #include "gtkprivate.h"
 #include "gtkrecentmanager.h"
-#include "gtksettingsprivate.h"
 #include "gtktooltipprivate.h"
 #include "gtkwidgetprivate.h"
 #include "gtkwindowprivate.h"
 #include "gtkwindowgroup.h"
-#include "gtkprintbackendprivate.h"
+#include "print/gtkprintbackendprivate.h"
 #include "gtkimmoduleprivate.h"
 #include "gtkroot.h"
 #include "gtknative.h"
@@ -95,10 +96,12 @@ typedef struct {
 #define N_DEBUG_DISPLAYS 4
 
 DisplayDebugFlags debug_flags[N_DEBUG_DISPLAYS];
+
 /* This is a flag to speed up development builds. We set it to TRUE when
  * any of the debug displays has debug flags >0, but we never set it back
  * to FALSE. This way we don't need to call gtk_widget_get_display() in
- * hot paths. */
+ * hot paths.
+ */
 gboolean any_display_debug_flags_set = FALSE;
 
 GtkDebugFlags
@@ -177,12 +180,6 @@ gtk_set_debug_flags (GtkDebugFlags flags)
   gtk_set_display_debug_flags (gdk_display_get_default (), flags);
 }
 
-gboolean
-gtk_simulate_touchscreen (void)
-{
-  return (gtk_get_debug_flags () & GTK_DEBUG_TOUCHSCREEN) != 0;
-}
-
 static const GdkDebugKey gtk_debug_keys[] = {
   { "keybindings", GTK_DEBUG_KEYBINDINGS, "Information about keyboard shortcuts" },
   { "modules", GTK_DEBUG_MODULES, "Information about modules and extensions" },
@@ -199,10 +196,10 @@ static const GdkDebugKey gtk_debug_keys[] = {
   { "builder-objects", GTK_DEBUG_BUILDER_OBJECTS, "Log unused GtkBuilder objects" },
   { "no-css-cache", GTK_DEBUG_NO_CSS_CACHE, "Disable style property cache" },
   { "interactive", GTK_DEBUG_INTERACTIVE, "Enable the GTK inspector", TRUE },
-  { "touchscreen", GTK_DEBUG_TOUCHSCREEN, "Pretend the pointer is a touchscreen" },
   { "snapshot", GTK_DEBUG_SNAPSHOT, "Generate debug render nodes" },
   { "accessibility", GTK_DEBUG_A11Y, "Information about accessibility state changes" },
   { "iconfallback", GTK_DEBUG_ICONFALLBACK, "Information about icon fallback" },
+  { "invert-text-dir", GTK_DEBUG_INVERT_TEXT_DIR, "Invert the default text direction", TRUE },
 };
 
 /* This checks to see if the process is running suid or sgid
@@ -506,10 +503,8 @@ gettext_initialization (void)
   setlocale_initialization ();
 
   bindtextdomain (GETTEXT_PACKAGE, _gtk_get_localedir ());
-  bindtextdomain (GETTEXT_PACKAGE "-properties", _gtk_get_localedir ());
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-  bind_textdomain_codeset (GETTEXT_PACKAGE "-properties", "UTF-8");
 #endif
 }
 
@@ -523,6 +518,7 @@ static void
 do_post_parse_initialization (void)
 {
   GdkDisplayManager *display_manager;
+  GtkTextDirection text_dir;
   gint64 before G_GNUC_UNUSED;
 
   if (gtk_initialized)
@@ -536,7 +532,16 @@ do_post_parse_initialization (void)
   signal (SIGPIPE, SIG_IGN);
 #endif
 
-  gtk_widget_set_default_direction (gtk_get_locale_direction ());
+  text_dir = gtk_get_locale_direction ();
+
+  /* We always allow inverting the text direction, even in
+   * production builds, as SDK/IDE tooling may provide the
+   * ability for developers to test rtl/ltr inverted.
+   */
+  if (gtk_get_debug_flags () & GTK_DEBUG_INVERT_TEXT_DIR)
+    text_dir = (text_dir == GTK_TEXT_DIR_LTR) ? GTK_TEXT_DIR_RTL : GTK_TEXT_DIR_LTR;
+
+  gtk_widget_set_default_direction (text_dir);
 
   gdk_event_init_types ();
 
@@ -622,34 +627,31 @@ gtk_init_check (void)
  * gtk_init:
  *
  * Call this function before using any other GTK functions in your GUI
- * applications.  It will initialize everything needed to operate the
+ * applications. It will initialize everything needed to operate the
  * toolkit.
  *
- * If you are using `GtkApplication`, you don't have to call gtk_init()
- * or gtk_init_check(); the `GApplication::startup` handler
- * does it for you.
+ * If you are using `GtkApplication`, you usually don't have to call this
+ * function; the `GApplication::startup` handler does it for you. Though,
+ * if you are using GApplication methods that will be invoked before `startup`,
+ * such as `local_command_line`, you may need to initialize stuff explicitly.
  *
  * This function will terminate your program if it was unable to
  * initialize the windowing system for some reason. If you want
- * your program to fall back to a textual interface you want to
- * call gtk_init_check() instead.
+ * your program to fall back to a textual interface, call
+ * [func@Gtk.init_check] instead.
  *
- * GTK calls `signal (SIGPIPE, SIG_IGN)`
- * during initialization, to ignore SIGPIPE signals, since these are
- * almost never wanted in graphical applications. If you do need to
- * handle SIGPIPE for some reason, reset the handler after gtk_init(),
- * but notice that other libraries (e.g. libdbus or gvfs) might do
- * similar things.
+ * GTK calls `signal (SIGPIPE, SIG_IGN)` during initialization, to ignore
+ * SIGPIPE signals, since these are almost never wanted in graphical
+ * applications. If you do need to handle SIGPIPE for some reason, reset
+ * the handler after gtk_init(), but notice that other libraries (e.g.
+ * libdbus or gvfs) might do similar things.
  */
 void
 gtk_init (void)
 {
   if (!gtk_init_check ())
     {
-      const char *display_name_arg = NULL;
-      if (display_name_arg == NULL)
-        display_name_arg = getenv ("DISPLAY");
-      g_warning ("cannot open display: %s", display_name_arg ? display_name_arg : "");
+      g_warning ("Failed to open display");
       exit (1);
     }
 }
@@ -719,8 +721,9 @@ gtk_init_check_abi_check (int num_checks, size_t sizeof_GtkWindow, size_t sizeof
 /**
  * gtk_is_initialized:
  *
- * Use this function to check if GTK has been initialized with gtk_init()
- * or gtk_init_check().
+ * Use this function to check if GTK has been initialized.
+ *
+ * See [func@Gtk.init].
  *
  * Returns: the initialization status
  */
@@ -757,30 +760,46 @@ gtk_is_initialized (void)
  * update_locale (const char *new_locale)
  * {
  *   setlocale (LC_ALL, new_locale);
- *   GtkTextDirection direction = gtk_get_locale_direction ();
- *   gtk_widget_set_default_direction (direction);
+ *   gtk_widget_set_default_direction (gtk_get_locale_direction ());
  * }
  * ]|
  *
- * Returns: the `GtkTextDirection` of the current locale
+ * Returns: the direction of the current locale
  */
 GtkTextDirection
 gtk_get_locale_direction (void)
 {
-  /* Translate to default:RTL if you want your widgets
-   * to be RTL, otherwise translate to default:LTR.
-   * Do *not* translate it to "predefinito:LTR", if it
-   * it isn't default:LTR or default:RTL it will not work
-   */
-  char             *e   = _("default:LTR");
-  GtkTextDirection  dir = GTK_TEXT_DIR_LTR;
+  PangoLanguage *language;
+  const PangoScript *scripts;
+  int n_scripts;
 
-  if (g_strcmp0 (e, "default:RTL") == 0)
-    dir = GTK_TEXT_DIR_RTL;
-  else if (g_strcmp0 (e, "default:LTR") != 0)
-    g_warning ("Whoever translated default:LTR did so wrongly. Defaulting to LTR.");
+  language = gtk_get_default_language ();
+  scripts = pango_language_get_scripts (language, &n_scripts);
 
-  return dir;
+  if (n_scripts > 0)
+    {
+      for (int i = 0; i < n_scripts; i++)
+        {
+          hb_script_t script;
+
+          script = hb_glib_script_to_script ((GUnicodeScript) scripts[i]);
+
+          switch (hb_script_get_horizontal_direction (script))
+            {
+            case HB_DIRECTION_LTR:
+              return GTK_TEXT_DIR_LTR;
+            case HB_DIRECTION_RTL:
+              return GTK_TEXT_DIR_RTL;
+            case HB_DIRECTION_TTB:
+            case HB_DIRECTION_BTT:
+            case HB_DIRECTION_INVALID:
+            default:
+              break;
+            }
+        }
+    }
+
+  return GTK_TEXT_DIR_LTR;
 }
 
 /**
@@ -796,12 +815,10 @@ gtk_get_locale_direction (void)
  * locale. It determines, for example, whether GTK uses
  * the right-to-left or left-to-right text direction.
  *
- * This function is equivalent to
- * [func@Pango.Language.get_default].
+ * This function is equivalent to [func@Pango.Language.get_default].
  * See that function for details.
  *
- * Returns: (transfer none): the default language as a
- *   `PangoLanguage`
+ * Returns: (transfer none): the default language
  */
 PangoLanguage *
 gtk_get_default_language (void)
@@ -897,31 +914,11 @@ rewrite_event_for_surface (GdkEvent  *event,
 			   GdkSurface *new_surface)
 {
   GdkEventType type;
-  double x, y;
+  double x = -G_MAXDOUBLE, y = -G_MAXDOUBLE;
   double dx, dy;
 
   type = gdk_event_get_event_type (event);
 
-  switch ((guint) type)
-    {
-    case GDK_BUTTON_PRESS:
-    case GDK_BUTTON_RELEASE:
-    case GDK_MOTION_NOTIFY:
-    case GDK_TOUCH_BEGIN:
-    case GDK_TOUCH_UPDATE:
-    case GDK_TOUCH_END:
-    case GDK_TOUCH_CANCEL:
-    case GDK_TOUCHPAD_SWIPE:
-    case GDK_TOUCHPAD_PINCH:
-    case GDK_TOUCHPAD_HOLD:
-      gdk_event_get_position (event, &x, &y);
-      gdk_surface_translate_coordinates (gdk_event_get_surface (event), new_surface, &x, &y);
-      break;
-    default:
-      x = y = 0;
-      break;
-    }
-
   switch ((guint) type)
     {
     case GDK_BUTTON_PRESS:
@@ -982,6 +979,7 @@ rewrite_event_for_surface (GdkEvent  *event,
                                            gdk_touchpad_event_get_pinch_angle_delta (event));
     case GDK_TOUCHPAD_HOLD:
       return gdk_touchpad_event_new_hold (new_surface,
+                                          gdk_event_get_event_sequence (event),
                                           gdk_event_get_device (event),
                                           gdk_event_get_time (event),
                                           gdk_event_get_modifier_state (event),
@@ -1031,8 +1029,7 @@ rewrite_event_for_grabs (GdkEvent *event)
       display = gdk_event_get_display (event);
       device = gdk_event_get_device (event);
 
-      if (!gdk_device_grab_info (display, device, &grab_surface, &owner_events) ||
-          !owner_events)
+      if (!gdk_device_grab_info (display, device, &grab_surface, &owner_events))
         return NULL;
       break;
     default:
@@ -1042,11 +1039,24 @@ rewrite_event_for_grabs (GdkEvent *event)
   event_widget = gtk_get_event_widget (event);
   grab_widget = GTK_WIDGET (gtk_native_get_for_surface (grab_surface));
 
-  if (grab_widget &&
-      gtk_main_get_window_group (grab_widget) != gtk_main_get_window_group (event_widget))
-    return rewrite_event_for_surface (event, grab_surface);
-  else
+  if (!grab_widget)
     return NULL;
+
+  /* If owner_events was set, events in client surfaces get forwarded
+   * as normal, but we consider other window groups foreign surfaces.
+   */
+  if (owner_events &&
+      gtk_main_get_window_group (grab_widget) == gtk_main_get_window_group (event_widget))
+    return NULL;
+
+  /* If owner_events was not set, events only get sent to the grabbing
+   * surface.
+   */
+  if (!owner_events &&
+      grab_surface == gtk_native_get_surface (gtk_widget_get_native (event_widget)))
+    return NULL;
+
+  return rewrite_event_for_surface (event, grab_surface);
 }
 
 static GdkEvent *
@@ -1078,7 +1088,8 @@ rewrite_event_for_toplevel (GdkEvent *event)
                             gdk_key_event_get_keycode (event),
                             gdk_event_get_modifier_state (event),
                             gdk_key_event_is_modifier (event),
-                            key, key_no_lock);
+                            key, key_no_lock,
+                            gdk_key_event_get_compose_sequence (event));
 }
 
 static gboolean
@@ -1180,6 +1191,11 @@ gtk_synthesize_crossing_events (GtkRoot         *toplevel,
   for (widget = new_target; widget; widget = _gtk_widget_get_parent (widget))
     gtk_widget_stack_append (&target_array, g_object_ref (widget));
 
+  if (old_target && new_target)
+    ancestor = gtk_widget_common_ancestor (old_target, new_target);
+  else
+    ancestor = NULL;
+
   crossing.direction = GTK_CROSSING_IN;
 
   seen_ancestor = FALSE;
