@@ -34,7 +34,14 @@
  * empty area, or by dragging the handle.
  *
  * `GtkSwitch` can also handle situations where the underlying state
- * changes with a delay. See [signal@GtkSwitch::state-set] for details.
+ * changes with a delay. In this case, the slider position indicates
+ * the user's recent change (as indicated by the [property@Gtk.Switch:active]
+ * property), and the color indicates whether the underlying state (represented
+ * by the [property@Gtk.Switch:state] property) has been updated yet.
+ *
+ * ![GtkSwitch with delayed state change](switch-state.png)
+ *
+ * See [signal@Gtk.Switch::state-set] for details.
  *
  * # CSS nodes
  *
@@ -246,7 +253,7 @@ gtk_switch_pan_gesture_pan (GtkGesturePan   *gesture,
 
   if (self->is_active)
     offset += width / 2;
-  
+
   offset /= width / 2;
   /* constrain the handle within the trough width */
   self->handle_pos = CLAMP (offset, 0, 1.0);
@@ -301,13 +308,14 @@ gtk_switch_measure (GtkWidget      *widget,
   GtkSwitch *self = GTK_SWITCH (widget);
   int slider_minimum, slider_natural;
   int on_nat, off_nat;
+  int on_baseline, off_baseline;
 
   gtk_widget_measure (self->slider, orientation, -1,
                       &slider_minimum, &slider_natural,
                       NULL, NULL);
 
-  gtk_widget_measure (self->on_image, orientation, for_size, NULL, &on_nat, NULL, NULL);
-  gtk_widget_measure (self->off_image, orientation, for_size, NULL, &off_nat, NULL, NULL);
+  gtk_widget_measure (self->on_image, orientation, for_size, NULL, &on_nat, NULL, &on_baseline);
+  gtk_widget_measure (self->off_image, orientation, for_size, NULL, &off_nat, NULL, &off_baseline);
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
     {
@@ -318,8 +326,12 @@ gtk_switch_measure (GtkWidget      *widget,
   else
     {
       int text_height = MAX (on_nat, off_nat);
+
       *minimum = MAX (slider_minimum, text_height);
       *natural = MAX (slider_natural, text_height);
+
+      *minimum_baseline = MAX (on_baseline, off_baseline) + MAX ((slider_minimum - text_height) / 2, 0);
+      *natural_baseline = MAX (on_baseline, off_baseline) + MAX ((slider_natural - text_height) / 2, 0);
     }
 }
 
@@ -526,7 +538,7 @@ gtk_switch_class_init (GtkSwitchClass *klass)
    *
    * The backend state that is controlled by the switch.
    *
-   * See [signal@GtkSwitch::state-set] for details.
+   * See [signal@Gtk.Switch::state-set] for details.
    */
   switch_props[PROP_STATE] =
     g_param_spec_boolean ("state", NULL, NULL,
@@ -742,9 +754,8 @@ gtk_switch_get_active (GtkSwitch *self)
  *
  * Sets the underlying state of the `GtkSwitch`.
  *
- * Normally, this is the same as [property@Gtk.Switch:active], unless
- * the switch is set up for delayed state changes. This function is
- * typically called from a [signal@Gtk.Switch::state-set] signal handler.
+ * This function is typically called from a [signal@Gtk.Switch::state-set]
+ * signal handler in order to set up delayed state changes.
  *
  * See [signal@Gtk.Switch::state-set] for details.
  */
@@ -761,12 +772,6 @@ gtk_switch_set_state (GtkSwitch *self,
 
   self->state = state;
 
-  /* This will be a no-op if we're switching the state in response
-   * to a UI change. We're setting active anyway, to catch 'spontaneous'
-   * state changes.
-   */
-  gtk_switch_set_active (self, state);
-
   if (state)
     gtk_widget_set_state_flags (GTK_WIDGET (self), GTK_STATE_FLAG_CHECKED, FALSE);
   else
